 include "settings.inc"
  include "../ti83plus.inc"
 include "equates.inc" 
 SEGMENT Main
 GLOBALS ON
 EXTERN USBErrorNoDevice,IsDeviceConnected,USBErrorInit,USBErrorClass,CDBuf,USBReqData
 EXTERN USBErrorAutoSetup,GetClass,AutoSetup,NegHL,USBHostInit,DispHexA,ReadDescriptor,PopRetC,RetC
 EXTERN FindDescriptor,PopErrorReadDescriptor,PopErrorFindDescriptor, AutoSetupNoInit,IPutC
 EXTERN GetIDs,USBErrorID,SendData,SetCallBack,SendControlOut,DoAssert,SendControlOutSafe,AutoSetupSafe
 EXTERN GetClassSafe,KillUSBDevice,USBErrorFeatureDisabled




SilverInit:
 IF SILVER_ON = 0
       jp     USBErrorFeatureDisabled  
 ELSE
       call   SilverInitDo
       ret    nc
       call   KillUSBDevice
       scf
       ret
 ENDIF


SilverSend:
;Send data to silver link       
;Input
; B = # of bytes to send, up to a maximum of 32
; HL -> data to send
;Output
; NC = success
 IF SILVER_ON = 0
       ret
 ELSE
       ld     c,2
       jp     SendData
 ENDIF

 IF SILVER_ON = 1
SilverInitDo:
;Initialize silver link driver
;Input
; DE -> descriptor buf
; ix -> callback routine
;Output
; NC = success
       ex     de,hl
       push   ix
       pop    de
       push   hl
       call   AutoSetup
       pop    hl
       jp     c,USBErrorAutoSetup

       call   GetIDs
       ret    c
       ld     bc,0451h
       or     a
       sbc    hl,bc
       jp     nz,USBErrorID
       ld     hl,0E001h
       or     a
       sbc    hl,de
       jp     nz,USBErrorID
       ld     b,1
       call   USBReqData
       xor    a
       ret
 ENDIF

PadVersion:
;Output
; DE = memory required by pad buffer
; H = major verison
; L = minor version
       ld     de,padMemSize
       ld     hl,PVersion
       ret

PadInit:
 IF PAD_ON = 0
       jp     USBErrorFeatureDisabled  
 ELSE
       call   PadInitDo
       ret    nc
       call   KillUSBDevice
       scf
       ret

PadInitDo:
;Initialize the gamepad controller driver and USB host
;Input
; HL-> Controller driver's memory
; IX-> CallBack routine
; DE-> Temporary buffer for descriptors
;Output
; NC = success
       LOG    PadInit
       push   ix
       pop    bc
       ld     ix,(MemPtr)
       ld     (ix+offPadBuf),l
       ld     (ix+offPadBuf+1),h
       push   de                   ;save temp buf
       ld     de,offpadCallBack
       add    hl,de
       ld     (hl),c
       inc    hl
       ld     (hl),b               ;Save callback
       pop    hl

       ld     de,PadCBWrapper
       push   hl                   ;Save desc buf loc
       call   AutoSetup
       pop    hl
       ret    c
       call   GetClass
       ret    c
       cp     3
       jr     nz,USBErrorClass     ;Not HID
       xor    a
       ret

PadCBWrapper:
       LOG    PadCallBack,b
       ld     hl,PadStartNoLog
       push   hl
       call   LdDEPadBuf
       ld     hl,offpadCallBack
       add    hl,de
       ld     a,(hl)
       inc    hl
       ld     h,(hl)
       ld     l,a
       ASSERTC h,9Dh
       ASSERTC 7Fh,h
       jp     (hl)

LdDEPadBuf:
       push   hl
       ld     hl,(MemPtr)
       ld     de,offPadBuf
       add    hl,de
       ld     e,(hl)
       inc    hl
       ld     d,(hl)
       pop    hl
       ret

 ENDIF



PadStart:
;Start passing along USB data
 IF PAD_ON = 0
       ret
 ELSE
       LOG    PadStart
PadStartNoLog:
       ld     b,1
       jp     USBReqData
 ENDIF


PadSetup:
;Get the offset and bit mask for a particular key
;This routine will be cancelled after a timeout or by an [on] key press
;Output
; NC = success
; B = offset byte
; C = bit mask
 IF PAD_ON = 0
       ret
 ELSE
       LOG    PadSetup
       ld     hl,(MemPtr)
       ld     de,offCallBackTable
       add    hl,de
       ld     e,(hl)
       inc    hl
       ld     d,(hl)
       push   de                          ;Save current callback
       ld     de,PadCallBack
       ld     (hl),d
       dec    hl
       ld     (hl),e                      ;Set PadSetup's callback
       call   LdDEPadBuf
       ld     b,padCheckBytes
       xor    a
$$:
       ld     (de),a
       inc    de
       djnz   $b
       ld     (de),a                      ;padFlags, No success initially

       ld     b,1
       call   USBReqData
       ld     b,0
PadSetupLoop:
       ei
       halt
       ld     a,(de)
       cp     2
       jr     z,PadSetupDone
       in     a,(4)
       bit    3,a
       jr     z,PadSetupFail
       djnz   PadSetupLoop
PadSetupFail:
       scf
PadSetupDone:
       pop    de
       push   af
       call   SetCallBack
       call   LdDEPadBuf
       ld     hl,offpadOffset
       add    hl,de
       ld     b,(hl)
       inc    hl
       ld     c,(hl)
       pop    af
       ret

PadCallBack:
       call   LdDEPadBuf
       push   de
       ld     hl,offpadFlags
       add    hl,de
       ld     a,(hl)
       or     a
       pop    de                   ;DE->PadBuf
       jr     nz,PadCBSecondTime
       inc    (hl)
       ld     hl,(MemPtr)
       ld     bc,padCheckBytes
       ldir
       ld     b,1
       jp     USBReqData

PadCBSecondTime:

       ld     hl,(MemPtr)
       ld     b,padCheckBytes
PadCBLoop:
       ld     a,(de)
       xor    (hl)
       jr     nz,PadCBDiff
PadCBLoopCont:
       inc    hl
       inc    de
       djnz   PadCBLoop
PadCBDone:
       ld     b,1
       jp     USBReqData

PadCBDiff:
       ld     c,a                         ;Save bit mask
       push   bc
       ld     b,8
       xor    a
PadCBDiffLoop:
       rr     c
       adc    a,0
       djnz   PadCBDiffLoop
       cp     1                           ;If more than one bit changed...
       pop    bc
       jr     nz,PadCBLoopCont            ; then we haven't found what we're looking for
       call   LdDEPadBuf
       ld     hl,offpadFlags
       add    hl,de
       ld     (hl),2                      ;Flag that we detected a button
       inc    hl
       ld     a,padCheckBytes
       sub    b
       ld     (hl),a                      ;Offset into byte array
       inc    hl
       ld     (hl),c                      ;Bit mask
       ret
 ENDIF





MouseInit:
 IF MOUSE_ON = 0
       jp     USBErrorFeatureDisabled  
 ELSE
       call   MouseInitDo
       ret    nc
       call   KillUSBDevice
       scf
       ret

MouseInitDo:
;Initialize mouse driver and USB host
;Input
; DE -> descriptor buf
; IX -> callback
; B = flags
;  bit 0 - set = allow diagonal keys, reset = disallow
;  bit 1 - set = call user's callback when mouse data arrives
;Output 
; NC = success
       push   de
       push   ix
       ld     a,b
       ld     hl,(MemPtr)
       ld     de,offMouseFlags
       add    hl,de
       ld     (hl),a
       inc    hl
       ld     bc,offMouseCallBack-offMouseBtn
       B_CALL MemClear
       ld     hl,(MemPtr)
       ld     de,offMouseCallBack
       add    hl,de
       pop    bc                   ;bc = callback addr
       ld     (hl),c               ; and HL-> mouseCallBack
       inc    hl
       ld     (hl),b               ;Save callback
       call   IsDeviceConnected
       pop    hl
       jp     z,USBErrorNoDevice
       push   hl
       call   USBHostInit
       pop    hl
       jp     c,USBErrorInit
       push   hl
       call   GetClass
       pop    hl
       jr     c,USBErrorClass
       cp     3
       jp     nz,USBErrorClass
       dec    b
       jp     nz,USBErrorClass
       dec    c
       dec    c
       jp     nz,USBErrorClass
       ld     de,MouseCallBack
       call   AutoSetupNoInit
       jp     c,USBErrorAutoSetup

       ld     b,1
       call   USBReqData
       xor    a
       ret
 ENDIF

MouseGetKey:
;Make sure mouse is intialized first, or this isn't going to return very useful data
;       DISAVE
 IF MOUSE_ON = 0
       ret
 ELSE
       ld     ix,(MemPtr)
       bit    mfSavedData,(ix+offMouseFlags)
       jr     z,$f
       res    mfSavedData,(ix+offMouseFlags)
       ld     b,(ix+offMouseBtn)
       ld     c,(ix+offMouseSavedData)
;       EIRESTORE
       ret
$$:
       ld     l,(ix+offMouseX)
       ld     h,(ix+offMouseX+1)
       bit    7,h
       push   af
       push   hl
       call   z,DoMouseRight
       pop    hl
       pop    af
       call   nz,DoMouseLeft
       ld     l,(ix+offMouseY)
       ld     h,(ix+offMouseY+1)
       bit    7,h
       push   af
       push   hl
       call   z,DoMouseDown
       pop    hl
       pop    af
       call   nz,DoMouseUp
       ld     a,(ix+offMouseFlags)
       xor    1<<mfSwitch
       ld     (ix+offMouseFlags),a
       bit    mfSwitch,(ix+offMouseFlags)
       jr     z,$f                 ;make sure that if both pressed and no diag, we treat fairly
       ld     a,b
       ld     b,c
       ld     c,a
$$:
       inc    c
       dec    c
       jr     nz,$f                ;make sure that if only one direciton is pressed, it's in c
       ld     a,b
       ld     b,c
       ld     c,a
$$:
       bit    mfDiag,(ix+offMouseFlags)
       jr     z,MouseGetKeyNoDiag
       ld     a,b
       or     c
       ld     c,a
       ld     b,(ix+offMouseBtn)
;       EIRESTORE
       ret

MouseGetKeyNoDiag:
       ld     a,b
       or     a
       ld     b,(ix+offMouseBtn)
       jr     z,$f
       set    mfSavedData,(ix+offMouseFlags)
       ld     (ix+offMouseSavedData),a
$$:
;       EIRESTORE
       ret

DoMouseLeft:
       call   NegHL
       call   CheckMouseThreshold
       ld     c,0
       ret    c
       ld     c,mouseLeft
       call   NegHL
MouseXCont:
       ld     (ix+offMouseX),l
       ld     (ix+offMouseX+1),h
       ret
DoMouseRight:
       call   CheckMouseThreshold
       ld     c,0
       ret    c
       ld     c,mouseRight
       jr     MouseXCont

DoMouseUp:
       call   NegHL
       call   CheckMouseThreshold
       ld     b,0
       ret    c
       ld     b,mouseUp
       call   NegHL
MouseYCont:
       ld     (ix+offMouseY),l
       ld     (ix+offMouseY+1),h
       ret
DoMouseDown:
       call   CheckMouseThreshold
       ld     b,0
       ret    c
       ld     b,mouseDown
       jr     MouseYCont

CheckMouseThreshold:
       ld     de,mouseThreshold
       or     a
       sbc    hl,de
       ret



MouseCallBack:
       ld     a,b
       cp     cbNewData
       ret    nz
       push   ix
       ld     hl,(MemPtr)
       push   hl
       pop    ix
       ld     de,offMouseBtn
       add    hl,de
       ld     a,(ix)
       ld     (hl),a
       inc    hl
       inc    ix
       call   AddMouseVal
       call   AddMouseVal
       pop    ix            ;Executing in interrupt.  Shouldn't mod IX
       ld     hl,(MemPtr)
       ld     de,offMouseFlags
       add    hl,de
       bit    1,(hl)
       jr     z,$f          ;Don't call callback
       ld     de,offMouseCallBack-offMouseFlags
       add    hl,de
       ld     e,(hl)
       inc    hl
       ld     d,(hl)
       push   de
$$:
       ld     b,1
       jp     USBReqData
                                          


AddMouseVal:
       ld     e,(hl)
       inc    hl
       ld     d,(hl)
       ex     de,hl                ;HL = MouseX/Y value

       ld     a,(ix)
       ld     c,a
       ld     b,0
       bit    7,a
       jr     z,$f
       dec    b
$$:
       add    hl,bc
       ex     de,hl
       ld     (hl),d
       dec    hl
       ld     (hl),e
       inc    hl
       inc    hl
       inc    ix
       ret       
 ENDIF

 COMMENT ~

GetHIDReportDesc:
;Get the HID report descriptor.  Assumes device already initialized
;Input
; HL -> descriptor buf (must be big enough for config descriptor)
;Output
; B = descriptor length
; NC = success
       push   hl
       push   hl
       ld     b,descConfig
       call   ReadDescriptor
       pop    hl
       jr     c,PopErrorReadDescriptor
       ld     b,descHID
       call   FindDescriptor
       jr     c,PopErrorFindDescriptor
       ld     de,6
       add    hl,de
       ld     c,(hl)               ;Descriptor type
       inc    hl
       ld     b,(hl)               ;Descriptor length
       push   bc
       ld     hl,HIDReportDesData
       rst    20h
       pop    bc
       ld     a,c
       ld     (OP1+3),a
       ld     a,b
       ld     (OP1+6),a
       pop    de
       push   bc
       ld     c,b
       ld     hl,OP1
       ld     b,descReport
       call   ReadDescriptor
       pop    bc
       ret

HIDReportDesData:
       db     81h,06,00,00,00,00,00,00
 ~
